#include <IRremote.h>

const int KEY_UP = 66;
const int KEY_DOWN = 67;
const int KEY_LEFT = 22;
const int KEY_RIGHT = 21;

const int KEY_STOP = 129;
const int KEY_DEBUG = 23;
const int KEY_SPEED_UP = 10;
const int KEY_SPEED_DOWN = 11;

const int MAX_SPEED_LEVEL = 10;
const int MIN_SPEED_LEVEL = 1;

const int MAX_SPEED = 200;
// const int DIRECTION_SPEED = MAX_SPEED / MAX_SPEED_LEVEL;

const int ENGINE_1 = 10;
const int ENGINE_2 = 5;

const int ENGINE_1_PIN_1 = 9;
const int ENGINE_1_PIN_2 = 8;
const int ENGINE_2_PIN_3 = 7;
const int ENGINE_2_PIN_4 = 6;

const int IR_RECEIVE_PIN = 11;

// State Vars
bool debug = true;
int actualSpeed = MAX_SPEED_LEVEL; // Always equal and more than MIN_SPEED_LEVEL and less or equal than MAX_SPEED_LEVEL

int calculateSpeed()
{
    // return MAX_SPEED / actualSpeed;
    return MAX_SPEED;
}

void moveEngine(int engine, int pinLow, int pinHigh, int speed)
{
    digitalWrite(pinLow, LOW);
    digitalWrite(pinHigh, HIGH);
    analogWrite(engine, speed);
}

void moveFoward()
{
    if (debug)
    {
        Serial.println("Foward");
    }
    moveEngine(ENGINE_1, ENGINE_1_PIN_1, ENGINE_1_PIN_2, calculateSpeed());
    moveEngine(ENGINE_2, ENGINE_2_PIN_3, ENGINE_2_PIN_4, calculateSpeed());
}

void moveBack()
{
    if (debug)
    {
        Serial.println("Back");
    }
    moveEngine(ENGINE_1, ENGINE_1_PIN_2, ENGINE_1_PIN_1, calculateSpeed());
    moveEngine(ENGINE_2, ENGINE_2_PIN_4, ENGINE_2_PIN_3, calculateSpeed());
}

void moveRight()
{
    if (debug)
    {
        Serial.println("Right");
    }
    moveEngine(ENGINE_1, ENGINE_1_PIN_1, ENGINE_1_PIN_2, calculateSpeed());
    moveEngine(ENGINE_2, ENGINE_2_PIN_4, ENGINE_2_PIN_3, calculateSpeed());
}

void moveLef()
{
    if (debug)
    {
        Serial.println("Left");
    }
    moveEngine(ENGINE_1, ENGINE_1_PIN_2, ENGINE_1_PIN_1, calculateSpeed());
    moveEngine(ENGINE_2, ENGINE_2_PIN_3, ENGINE_2_PIN_4, calculateSpeed());
}

void dance() {
    if (debug)
    {
        Serial.println("Lets dance!");
    }
    moveRight();
    delay(200);
    moveLef();
    delay(200);
}

void stop()
{
    if (debug)
    {
        Serial.println("Stop");
    }
    digitalWrite(ENGINE_1_PIN_1, LOW);
    digitalWrite(ENGINE_1_PIN_2, LOW);
    digitalWrite(ENGINE_2_PIN_3, LOW);
    digitalWrite(ENGINE_2_PIN_4, LOW);
}

void setup()
{
    Serial.begin(9600);
    IrReceiver.begin(IR_RECEIVE_PIN, ENABLE_LED_FEEDBACK);
    pinMode(ENGINE_1, OUTPUT);
    pinMode(ENGINE_2, OUTPUT);
    pinMode(ENGINE_1_PIN_1, OUTPUT);
    pinMode(ENGINE_1_PIN_2, OUTPUT);
    pinMode(ENGINE_2_PIN_3, OUTPUT);
    pinMode(ENGINE_2_PIN_4, OUTPUT);
}

void loop()
{
    if (IrReceiver.decode())
    {
        int command = IrReceiver.decodedIRData.command;
        switch (command)
        {
        case KEY_UP:
            moveFoward();
            break;
        case KEY_DOWN:
            moveBack();
            break;
        case KEY_LEFT:
            moveLef();
            break;
        case KEY_RIGHT:
            moveRight();
            break;
        case KEY_STOP:
            stop();
            break;
        case KEY_SPEED_UP:
            actualSpeed++;
            if (actualSpeed > MAX_SPEED_LEVEL)
            {
                actualSpeed = MAX_SPEED_LEVEL;
            }
            Serial.print("Actual speed is ");
            Serial.println(actualSpeed);
            break;
        case KEY_SPEED_DOWN:
            actualSpeed--;
            if (actualSpeed < MIN_SPEED_LEVEL)
            {
                actualSpeed = MIN_SPEED_LEVEL;
            }
            Serial.print("Actual speed is ");
            Serial.println(actualSpeed);
            break;
        case KEY_DEBUG:
            debug = !debug;
            Serial.print("Debug: ");
            Serial.println(debug ? "Enabled" : "Disabled");
            break;
        default:
            Serial.print("Unknown command: ");
            Serial.println(command, DEC);
            break;
        }
        delay(50); // Skiop doble input
        IrReceiver.resume();
    }
}
